import pandas as pd

from experiments.shared.latex import table_from_df


def create_table(df_mean, df_std):
    df_mean.columns = [c[1] for c in df_mean.columns.to_list()]
    df_std.columns = [c[1] for c in df_std.columns.to_list()]

    # Swap the order of Synthetic and Libra columns
    cols = list(df_mean.columns)
    cols = cols[3:6] + cols[0:3]
    df_mean = df_mean[cols]

    cols = list(df_std.columns)
    cols = cols[3:6] + cols[0:3]
    df_std = df_std[cols]

    table_string = table_from_df(df_mean, df_std)
    return table_string


# Sorts by Libra 50, then 20, then 1
def sort_rows_by_libra_datasets(df):
    df_sorted = df.sort_values(
        ["libra 50", "libra 20", "libra 1"], ascending=[True, True, True]
    )
    return df_sorted


if __name__ == "__main__":
    # NOTE: Select results path
    csv_path = "transformer_results/20220526/transformer_25-05-2022_20:46:02.csv"
    df = pd.read_csv(csv_path)

    df_mae = df[["transformer_type", "dataset_horizon", "mae"]]
    df_rmse = df[["transformer_type", "dataset_horizon", "rmse"]]

    df_mae_mean = (
        df_mae.groupby(["transformer_type", "dataset_horizon"]).mean().unstack()
    )
    df_mae_std = df_mae.groupby(["transformer_type", "dataset_horizon"]).std().unstack()
    df_rmse_mean = (
        df_rmse.groupby(["transformer_type", "dataset_horizon"]).mean().unstack()
    )
    df_rmse_std = (
        df_rmse.groupby(["transformer_type", "dataset_horizon"]).std().unstack()
    )

    table_mae = create_table(df_mae_mean, df_mae_std)
    table_rmse = create_table(df_rmse_mean, df_rmse_std)

    print(">>> Table MAE")
    print(table_mae + "\n\n")
    print(">>> Table RMSE")
    print(table_rmse + "\n\n")
