import numpy as np
import torch
from matplotlib import pyplot as plt
from numpy.lib.stride_tricks import sliding_window_view
from torch import nn

from experiments.shared.utils import save_current_figure
from src.transformers.encoder_decoder.train_encoder_decoder_informer import (
    make_prediction as make_prediction_edi,
)
from src.transformers.encoder_decoder.train_encoder_decoder import (
    make_prediction as make_prediction_ed,
)
from src.transformers.encoder_only.train_encoder_only import (
    make_prediction as make_prediction_eo,
)
from src.transformers.decoder_only.train_decoder_only import (
    make_prediction as make_prediction_do,
)
from experiments.e1.src.evaluation import TransformerType


def make_prediction(
    model: nn.Module,
    transformer_type: TransformerType,
    source: np.ndarray,
    device,
    input_dim=None,
    output_dim=None,
    patch_size=None,
    split_size=None,
):
    model = model.to(device=device)
    model.eval()

    source_tensor = torch.Tensor(source).unsqueeze(0).to(device=device)
    if transformer_type == TransformerType.ENCODER_DECODER_INFORMER:
        prediction_tensor = make_prediction_edi(
            model, source_tensor, output_dim, split_size
        )
    elif transformer_type == TransformerType.ENCODER_DECODER:
        prediction_tensor = make_prediction_ed(
            model, source_tensor, output_dim, patch_size
        )
    elif transformer_type == TransformerType.DECODER_ONLY:
        prediction_tensor = make_prediction_do(
            model, source_tensor, output_dim, patch_size
        )
    elif (
        transformer_type == TransformerType.ENCODER_ONLY
        or transformer_type == TransformerType.ENCODER_ONLY_SIMPLE
    ):
        use_mask = patch_size == 1
        prediction_tensor = make_prediction_eo(
            model, source_tensor, input_dim, use_mask
        )

    prediction = prediction_tensor.squeeze(0).detach().cpu().numpy()
    return prediction


def sliding_window(values, window_size=3):
    smoothed = sliding_window_view(np.array(values), window_shape=window_size)
    return smoothed


def simple_plot(y):
    x = np.arange(0, len(y))
    plt.plot(x, y)
    plt.show()


def plot_prediction(source, target, prediction, png_path=None):
    source_length = len(source)
    target_length = len(target)
    x_source = np.arange(0, source_length)
    x_target = np.arange(source_length, source_length + target_length)

    plt.plot(x_source, source)
    if target_length == 1:
        plt.plot(x_target, target, "o")
        plt.plot(x_target, prediction, "o")
    else:
        plt.plot(x_target, target)
        plt.plot(x_target, prediction)

    if png_path is not None:
        save_current_figure(png_path)

    plt.show()
