import pandas as pd

from experiments.shared.latex import table_from_df


def create_table(df_mean, df_std, print_sorted_attentions=False):
    df_mean.columns = [c[1] for c in df_mean.columns.to_list()]
    df_std.columns = [c[1] for c in df_std.columns.to_list()]

    # Swap the order of Synthetic and Libra columns
    cols = list(df_mean.columns)
    cols = cols[3:6] + cols[0:3]
    df_mean = df_mean[cols]

    cols = list(df_std.columns)
    cols = cols[3:6] + cols[0:3]
    df_std = df_std[cols]

    if print_sorted_attentions:
        print("### Sorted attentions ###")
        df_mean_sorted = sort_rows_by_libra_datasets(df_mean)
        attention_names = df_mean_sorted.iloc[:, -3:]  # Display Libra 400 column
        print(attention_names)
        print("")

    table_string = table_from_df(df_mean, df_std)
    return table_string


# Sorts by Libra 400, then 200, then 100
def sort_rows_by_libra_datasets(df):
    df_sorted = df.sort_values(
        ["libra 400", "libra 200", "libra 100"], ascending=[True, True, True]
    )
    return df_sorted


if __name__ == "__main__":
    # NOTE: Select dataset
    csv_path = "results/20220525/attention_23-05-2022_23:35:22.csv"
    df = pd.read_csv(csv_path)

    # Full encoder
    df_full_encoder = df[df["transformer_type"] == "encoder_only"]
    df_full_encoder_mae = df_full_encoder[["attention_type", "dataset_horizon", "mae"]]
    df_full_encoder_rmse = df_full_encoder[
        ["attention_type", "dataset_horizon", "rmse"]
    ]

    df_full_encoder_mae_mean = (
        df_full_encoder_mae.groupby(["attention_type", "dataset_horizon"])
        .mean()
        .unstack()
    )
    df_full_encoder_mae_std = (
        df_full_encoder_mae.groupby(["attention_type", "dataset_horizon"])
        .std()
        .unstack()
    )
    df_full_encoder_rmse_mean = (
        df_full_encoder_rmse.groupby(["attention_type", "dataset_horizon"])
        .mean()
        .unstack()
    )
    df_full_encoder_rmse_std = (
        df_full_encoder_rmse.groupby(["attention_type", "dataset_horizon"])
        .std()
        .unstack()
    )

    full_encoder_table_mae = create_table(
        df_full_encoder_mae_mean, df_full_encoder_mae_std
    )
    full_encoder_table_rmse = create_table(
        df_full_encoder_rmse_mean, df_full_encoder_rmse_std
    )

    print(">>> Full MAE")
    print(full_encoder_table_mae + "\n\n")
    print(">>> Full RMSE")
    print(full_encoder_table_rmse + "\n\n")

    # BERT encoder
    df_bert_encoder = df[df["transformer_type"] == "encoder_only_simple"]
    df_bert_encoder_mae = df_bert_encoder[["attention_type", "dataset_horizon", "mae"]]
    df_bert_encoder_rmse = df_bert_encoder[
        ["attention_type", "dataset_horizon", "rmse"]
    ]

    df_bert_encoder_mae_mean = (
        df_bert_encoder_mae.groupby(["attention_type", "dataset_horizon"])
        .mean()
        .unstack()
    )
    df_bert_encoder_mae_std = (
        df_bert_encoder_mae.groupby(["attention_type", "dataset_horizon"])
        .std()
        .unstack()
    )
    df_bert_encoder_rmse_mean = (
        df_bert_encoder_rmse.groupby(["attention_type", "dataset_horizon"])
        .mean()
        .unstack()
    )
    df_bert_encoder_rmse_std = (
        df_bert_encoder_rmse.groupby(["attention_type", "dataset_horizon"])
        .std()
        .unstack()
    )

    bert_encoder_table_mae = create_table(
        df_bert_encoder_mae_mean, df_bert_encoder_mae_std, print_sorted_attentions=True
    )
    bert_encoder_table_rmse = create_table(
        df_bert_encoder_rmse_mean,
        df_bert_encoder_rmse_std,
        print_sorted_attentions=True,
    )

    print(">>> BERT MAE")
    print(bert_encoder_table_mae + "\n\n")
    print(">>> BERT RMSE")
    print(bert_encoder_table_rmse + "\n\n")
