import pandas as pd
from experiments.shared.stats import complete_anova_run


if __name__ == "__main__":
    # NOTE: Select dataset
    csv_path = "results/20220525/attention_23-05-2022_23:35:22.csv"
    df = pd.read_csv(csv_path)

    # One way ANOVA, only test dependency on attention model
    # The p value obtained from ANOVA analysis is significant (p < 0.05)
    # Note: p-value 0.001 from tukey_hsd output should be interpreted as <=0.001
    # Example formula: anova_model = "value~C(Genotype)+C(years)+C(Genotype):C(years)"
    formula_mae = "mae ~ C(attention_type)"
    formula_rmse = "rmse ~ C(attention_type)"

    # Full encoder
    df_full_encoder = df[df["transformer_type"] == "encoder_only"]
    df_full_encoder_mae = df_full_encoder[["attention_type", "dataset_horizon", "mae"]]
    df_full_encoder_rmse = df_full_encoder[
        ["attention_type", "dataset_horizon", "rmse"]
    ]

    (
        anova_df_full_encoder_mae,
        tukey_df_full_encoder_mae,
        normality_full_encoder_mae,
        homogeneity_full_encoder_mae,
    ) = complete_anova_run(
        df_full_encoder_mae,
        formula_mae,
        "mae",
        "attention_type",
    )

    (
        anova_df_full_encoder_rmse,
        tukey_full_encoder_df_rmse,
        normality_full_encoder_rmse,
        homogeneity_full_encoder_rmse,
    ) = complete_anova_run(
        df_full_encoder_rmse,
        formula_rmse,
        "rmse",
        "attention_type",
    )

    print(">>> Full encoder")
    print(anova_df_full_encoder_mae)
    print(anova_df_full_encoder_rmse)

    # BERT encoder
    df_bert_encoder = df[df["transformer_type"] == "encoder_only_simple"]
    df_bert_encoder_mae = df_bert_encoder[["attention_type", "dataset_horizon", "mae"]]
    df_bert_encoder_rmse = df_bert_encoder[
        ["attention_type", "dataset_horizon", "rmse"]
    ]

    (
        anova_df_bert_encoder_mae,
        tukey_df_bert_encoder_mae,
        normality_bert_encoder_mae,
        homogeneity_bert_encoder_mae,
    ) = complete_anova_run(
        df_bert_encoder_mae,
        formula_mae,
        "mae",
        "attention_type",
    )

    (
        anova_df_bert_encoder_rmse,
        tukey_bert_encoder_df_rmse,
        normality_bert_encoder_rmse,
        homogeneity_bert_encoder_rmse,
    ) = complete_anova_run(
        df_bert_encoder_rmse,
        formula_rmse,
        "rmse",
        "attention_type",
    )

    print(">>> BERT encoder")
    print(anova_df_bert_encoder_mae)
    print(anova_df_bert_encoder_rmse)
