import pandas as pd
from experiments.shared.stats import complete_anova_run


if __name__ == "__main__":
    # NOTE: Select results csv
    csv_path = "results/all-modules-raw.csv"
    df = pd.read_csv(csv_path)

    # One way ANOVA, only test dependency on attention model
    # The p value obtained from ANOVA analysis is significant (p < 0.05)
    # Note: p-value 0.001 from tukey_hsd output should be interpreted as <=0.001
    # Example formula: anova_model = "value~C(Genotype)+C(years)+C(Genotype):C(years)"
    formula = "fitness ~ C(encoding_variant)"

    df_filtered = df[["encoding_variant", "fitness"]]

    # ANOVA: Significant if PR(>F) < 0.05
    # Tukey: Significant if p-value < 0.05
    (anova_df, tukey_df, normality, homogeneity) = complete_anova_run(
        df_filtered,
        formula,
        "fitness",
        "encoding_variant",
    )

    print(">>> Genotype encoding variants")
    print(anova_df)
