import pandas as pd

from experiments.shared.latex import table_from_df


def create_table(df_mean, df_std, df_mean2, df_std2, category):
    df_mean.columns = [c[1] for c in df_mean.columns.to_list()]
    df_std.columns = [c[1] for c in df_std.columns.to_list()]

    df_mean2.columns = [c[1] for c in df_mean2.columns.to_list()]
    df_std2.columns = [c[1] for c in df_std2.columns.to_list()]

    # Swap the order of Libra columns
    # cols = list(df_mean.columns)
    # cols = cols[3:6] + cols[0:3]
    # df_mean = df_mean[cols]

    # cols = list(df_std.columns)
    # cols = cols[3:6] + cols[0:3]
    # df_std = df_std[cols]

    table_string = table_from_df(
        df_mean, df_std, df_mean2, df_std2, first_column_header=category
    )
    return table_string


def preprocess_df_by_category(d, category):
    df_methods = df[df["category"] == category]
    df_methods_mae = df_methods[["method_type", "dataset_input_horizon", "mae"]]
    df_methods_rmse = df_methods[["method_type", "dataset_input_horizon", "rmse"]]

    df_methods_mae_mean = (
        df_methods_mae.groupby(["method_type", "dataset_input_horizon"])
        .mean()
        .unstack()
    )
    df_methods_mae_std = (
        df_methods_mae.groupby(["method_type", "dataset_input_horizon"]).std().unstack()
    )
    df_methods_rmse_mean = (
        df_methods_rmse.groupby(["method_type", "dataset_input_horizon"])
        .mean()
        .unstack()
    )
    df_methods_rmse_std = (
        df_methods_rmse.groupby(["method_type", "dataset_input_horizon"])
        .std()
        .unstack()
    )

    return (
        df_methods_mae_mean,
        df_methods_mae_std,
        df_methods_rmse_mean,
        df_methods_rmse_std,
    )


if __name__ == "__main__":
    # NOTE: Select dataset
    csv_path = "results/methods_merged.csv"
    df = pd.read_csv(csv_path)

    print(">>> Libra - Economics")
    category = "economics"
    (
        df_methods_mae_mean,
        df_methods_mae_std,
        df_methods_rmse_mean,
        df_methods_rmse_std,
    ) = preprocess_df_by_category(df, category=category)
    methods_table_mae = create_table(
        df_methods_mae_mean,
        df_methods_mae_std,
        df_methods_rmse_mean,
        df_methods_rmse_std,
        category,
    )
    print(methods_table_mae + "\n\n")

    print(">>> Libra - Finance")
    category = "finance"
    (
        df_methods_mae_mean,
        df_methods_mae_std,
        df_methods_rmse_mean,
        df_methods_rmse_std,
    ) = preprocess_df_by_category(df, category=category)
    methods_table_mae = create_table(
        df_methods_mae_mean,
        df_methods_mae_std,
        df_methods_rmse_mean,
        df_methods_rmse_std,
        category,
    )
    print(methods_table_mae + "\n\n")

    print(">>> Libra - Human")
    category = "human"
    (
        df_methods_mae_mean,
        df_methods_mae_std,
        df_methods_rmse_mean,
        df_methods_rmse_std,
    ) = preprocess_df_by_category(df, category=category)
    methods_table_mae = create_table(
        df_methods_mae_mean,
        df_methods_mae_std,
        df_methods_rmse_mean,
        df_methods_rmse_std,
        category,
    )
    print(methods_table_mae + "\n\n")

    print(">>> Libra - Nature")
    category = "nature"
    (
        df_methods_mae_mean,
        df_methods_mae_std,
        df_methods_rmse_mean,
        df_methods_rmse_std,
    ) = preprocess_df_by_category(df, category=category)
    methods_table_mae = create_table(
        df_methods_mae_mean,
        df_methods_mae_std,
        df_methods_rmse_mean,
        df_methods_rmse_std,
        category,
    )
    print(methods_table_mae + "\n\n")
