import pandas as pd
from experiments.shared.stats import complete_anova_run


if __name__ == "__main__":
    # NOTE: Select dataset
    csv_path = "results/methods_merged.csv"
    df = pd.read_csv(csv_path)

    # One way ANOVA, only test dependency on attention model
    # The p value obtained from ANOVA analysis is significant (p < 0.05)
    # Note: p-value 0.001 from tukey_hsd output should be interpreted as <=0.001
    # Example formula: anova_model = "value~C(Genotype)+C(years)+C(Genotype):C(years)"
    formula_mae = "mae ~ C(method_type)"
    formula_rmse = "rmse ~ C(method_type)"

    # Filter category if needed: finance, economics, human, nature
    # df = df[df["category"] == "finance"]

    # ANOVA: Significant if PR(>F) < 0.05
    # Tukey: Significant if p-value < 0.05
    df_methods_mae = df[["method_type", "dataset_input_horizon", "mae"]]
    df_methods_rmse = df[["method_type", "dataset_input_horizon", "rmse"]]

    (
        anova_df_methods_mae,
        tukey_df_methods_mae,
        normality_methods_mae,
        homogeneity_methods_mae,
    ) = complete_anova_run(
        df_methods_mae,
        formula_mae,
        "mae",
        "method_type",
    )

    (
        anova_df_methods_rmse,
        tukey_methods_df_rmse,
        normality_methods_rmse,
        homogeneity_methods_rmse,
    ) = complete_anova_run(
        df_methods_rmse,
        formula_rmse,
        "rmse",
        "method_type",
    )

    print(">>> Methods comparison")
    print(anova_df_methods_mae)
    print(anova_df_methods_rmse)
