import numpy as np
import torch


def batch_iterator(dataset, batch_size, device):
    for i in range(0, len(dataset) - batch_size, batch_size):
        batch = [dataset[j] for j in range(i, i + batch_size)]
        x = np.array([[s for s in b["x"]] for b in batch], dtype=np.dtype("f4"))
        y = np.array([[s for s in b["y"]] for b in batch], dtype=np.dtype("f4"))

        source = torch.tensor(x, requires_grad=True).to(device=device)
        target = torch.tensor(y, requires_grad=True).to(device=device)
        yield source, target


def create_prediction_batch(source, target, device):
    x = np.array(source, dtype=np.dtype("f4"))
    y = np.array(target, dtype=np.dtype("f4"))
    source = torch.tensor(x, requires_grad=True).to(device=device)
    target = torch.tensor(y, requires_grad=True).to(device=device)
    return [(source, target)]
