import copy
from enum import Enum
from typing import List

from src.components.activation.ELU import ELU
from src.components.activation.GeLU import GeLU
from src.components.activation.LeakyReLU import LeakyReLU
from src.components.activation.ReLU import ReLU
from src.components.activation.SiLU import SiLU
from src.components.activation.SquaredReLU import SquaredReLU


# Good candidates for Transformers are: GeLU, LeakyReLU, ReLU, SiLU (Swish)
class Activation(str, Enum):
    ELU = "elu"
    GeLU = "gelu"
    LeakyReLU = "leaky_relu"
    ReLU = "relu"
    SiLU = "silu"
    SquaredReLU = "squared_relu"

    @staticmethod
    def from_str(string: str):
        return Activation(string)

    @staticmethod
    def module_list() -> List[Enum]:
        return [activation for activation in Activation]

    @staticmethod
    def module_string_list() -> List[str]:
        return [activation.value for activation in Activation]

    def module(self):
        registry = {
            Activation.ELU: ELU(),
            Activation.GeLU: GeLU(),
            Activation.LeakyReLU: LeakyReLU(),
            Activation.ReLU: ReLU(),
            Activation.SiLU: SiLU(),
            Activation.SquaredReLU: SquaredReLU(),
        }
        return copy.deepcopy(registry[self])
