import copy
from enum import Enum
from typing import List

from torch import nn

from src.components.convolution.SpatialDepthWiseConvolution import (
    SpatialDepthWiseConvolution,
)
from src.components.positional_encoding.mam.AbsolutePositionalEncodingTupe import (
    AbsolutePositionalEncodingTupe,
)
from src.components.positional_encoding.mam.RotaryPositionalEncoding import (
    RotaryPositionalEncoding,
)
from src.components.transformer.Passthrough import Passthrough


class QKVPreprocess(str, Enum):
    NoPreprocess = "no_preprocess"
    RotaryPositionalEncoding = "rotary_positional_encoding"
    AbsolutePositionalEncodingTupe = "absolute_positional_encoding_tupe"
    SpatialDepthWiseConvolution = "spatial_depthwise_convolution"

    @staticmethod
    def from_str(string: str):
        return QKVPreprocess(string)

    @staticmethod
    def module_list() -> List[Enum]:
        return [preprocess for preprocess in QKVPreprocess]

    @staticmethod
    def module_string_list() -> List[str]:
        return [preprocess.value for preprocess in QKVPreprocess]

    def module(self, embedding_dim=None, head_count=None, d_k=None) -> nn.Module:
        registry = {
            QKVPreprocess.NoPreprocess: Passthrough(),
            QKVPreprocess.RotaryPositionalEncoding: RotaryPositionalEncoding(d_k),
            QKVPreprocess.SpatialDepthWiseConvolution: SpatialDepthWiseConvolution(d_k),
            QKVPreprocess.AbsolutePositionalEncodingTupe: AbsolutePositionalEncodingTupe(
                embedding_dim, head_count
            ),
        }
        return copy.deepcopy(registry[self])
