import copy
from enum import Enum
from typing import List

from torch import nn

from src.components.positional_encoding.mam.RelativePositionalEncodingDeBERTa import (
    RelativePositionalEncodingDeBERTa,
)
from src.components.positional_encoding.mam.RelativePositionalEncodingT5 import (
    RelativePositionalEncodingT5,
)
from src.components.positional_encoding.mam.RelativePositionalEncodingTupe import (
    RelativePositionalEncodingTupe,
)
from src.components.transformer.Passthrough import Passthrough


class ScoresPreprocess(str, Enum):
    NoPreprocess = "no_preprocess"
    RelativePositionalEncodingDeBERTa = "relative_positional_encoding_deberta"
    RelativePositionalEncodingT5 = "relative_positional_encoding_t5"
    RelativePositionalEncodingTupe = "relative_positional_encoding_tupe"

    @staticmethod
    def from_str(string: str):
        return ScoresPreprocess(string)

    @staticmethod
    def module_list() -> List[Enum]:
        return [preprocess for preprocess in ScoresPreprocess]

    @staticmethod
    def module_string_list() -> List[str]:
        return [preprocess.value for preprocess in ScoresPreprocess]

    def module(self, head_count=None) -> nn.Module:
        registry = {
            ScoresPreprocess.NoPreprocess: Passthrough(),
            ScoresPreprocess.RelativePositionalEncodingDeBERTa: RelativePositionalEncodingDeBERTa(),
            ScoresPreprocess.RelativePositionalEncodingT5: RelativePositionalEncodingT5(
                head_count
            ),
            ScoresPreprocess.RelativePositionalEncodingTupe: RelativePositionalEncodingTupe(
                head_count
            ),
        }
        return copy.deepcopy(registry[self])
