import unittest

import torch
from torch import nn


# NOTE: Dilation >1 turns the module into dilated convolution
class Conv1d(nn.Module):
    def __init__(self, kernel_size: int = 3, dilation: int = 1):
        super(Conv1d, self).__init__()

        # Adjust for even kernel sizes
        if (kernel_size % 2) == 0:
            kernel_size = kernel_size + 1

        self.conv1d = nn.Conv1d(
            1,
            1,
            kernel_size,
            stride=1,
            padding=(kernel_size // 2) * dilation,
            dilation=dilation,
        )

    # Run convolution along embedding_dim
    # It basically behaves like depth-wise convolution
    def forward(self, x):
        sequence_length = x.size(1)
        out = torch.zeros(x.shape, device=x.device)
        for i in range(sequence_length):
            conv_in = x[:, i, :].unsqueeze(1)
            conv_out = self.conv1d(conv_in)
            out[:, i, :] = conv_out.squeeze(1)
        return out


class TestConv1d(unittest.TestCase):
    def test(self):
        conv1d = Conv1d(16, 5)
        input = torch.randn(20, 100, 16)
        output = conv1d(input)
        self.assertEqual(list(output.shape), [20, 100, 16])
