import unittest

import torch
from torch import nn


class DepthWiseSeparableConv(nn.Module):
    def __init__(self, embedding_dim: int, kernel_size: int = 3, dilation: int = 1):
        super(DepthWiseSeparableConv, self).__init__()

        # Adjust for even kernel sizes
        if (kernel_size % 2) == 0:
            kernel_size = kernel_size + 1

        self.depth_wise = nn.Conv2d(
            embedding_dim,
            embedding_dim,
            groups=embedding_dim,
            kernel_size=kernel_size,
            padding=(kernel_size // 2) * dilation,
            dilation=dilation,
        )
        self.point_wise = nn.Conv2d(embedding_dim, embedding_dim, kernel_size=1)

    def forward(self, x):
        x = x.permute(0, 2, 1)
        x = x.unsqueeze(2)
        out = self.depth_wise(x)
        out = self.point_wise(out)
        out = out.squeeze(2)
        out = out.permute(0, 2, 1)
        return out


class TestDepthWiseSeparableConv(unittest.TestCase):
    def test(self):
        conv1d = DepthWiseSeparableConv(16, 6, 2)
        input = torch.randn(20, 100, 16)
        output = conv1d(input)
        self.assertEqual(list(output.shape), [20, 100, 16])
