import unittest

import torch
from torch import nn


# NOTE: Dilation >1 turns the module into dilated convolution
class PointWiseConv1d(nn.Module):
    def __init__(self, embedding_dim: int, kernel_size: int = 3, dilation: int = 1):
        super(PointWiseConv1d, self).__init__()

        # Adjust for even kernel sizes
        if (kernel_size % 2) == 0:
            kernel_size = kernel_size + 1

        self.conv1d = nn.Conv1d(
            embedding_dim,
            embedding_dim,
            kernel_size,
            stride=1,
            padding=(kernel_size // 2) * dilation,
            dilation=dilation,
        )

    def forward(self, x):
        conv_in = x.transpose(1, 2)
        conv_out = self.conv1d(conv_in)
        out = conv_out.transpose(1, 2)
        return out


class TestConv1d(unittest.TestCase):
    def test(self):
        conv1d = PointWiseConv1d(16, 5)
        input = torch.randn(20, 100, 16)
        output = conv1d(input)
        self.assertEqual(list(output.shape), [20, 100, 16])
