import torch
from torch import nn


class LearnedPositionalEncodingBERT(nn.Module):
    def __init__(self, embedding_dim, max_sequence_length=1024):
        super(LearnedPositionalEncodingBERT, self).__init__()
        self.positional_embedding = nn.Embedding(max_sequence_length, embedding_dim)

    def forward(self, x: torch.Tensor):
        # x: (batch_size, sequence_length, embedding_size)
        batch_size = x.size(0)
        sequence_length = x.size(1)
        pos = torch.arange(sequence_length, dtype=torch.long, device=x.device)
        pos = pos.unsqueeze(0).expand(
            batch_size, -1
        )  # (sequence_length,) -> (batch_size, sequence_length)
        embedding = self.positional_embedding(pos)
        return x + embedding
