import torch
from torch import nn

# TUPE positional encoding
# Adapted from: https://github.com/jaketae/tupe
# Paper: Rethinking Positional Encoding in Language Pre-training
from src.components.positional_encoding.ape.SinusoidalAbsolutePositionalEncoding import (
    SinusoidalAbsolutePositionalEncoding,
)


class AbsolutePositionalEncodingTupe(nn.Module):
    def __init__(self, embedding_dim: int, head_count: int):
        super().__init__()
        self.embedding_dim = embedding_dim
        self.head_count = head_count

        self.pos_embed = SinusoidalAbsolutePositionalEncoding(embedding_dim, 0.0)
        self.linear = nn.Linear(embedding_dim, embedding_dim, bias=False)

    def forward(self, x):
        # (batch_size, head_count, sequence_length, sequence_length)
        batch_size = x.size(0)
        sequence_length = x.size(2)
        pos_embed = self.pos_embed(
            torch.zeros(batch_size, sequence_length, 1, device=x.device)
        )
        # pos_embed.shape == (batch_size, seq_len, d_model)

        pos_embed = self.linear(pos_embed)
        pos_embed = pos_embed.view(batch_size, sequence_length, self.head_count, -1)
        pos_embed = pos_embed.permute(0, 2, 1, 3)

        return x + pos_embed
