import torch.nn as nn

from src.components.activation.Activation import Activation


class FeedForward(nn.Module):
    def __init__(
        self,
        embedding_dim,
        forward_expansion,
        dropout=0.1,
        activation: str = Activation.ReLU.value,
        is_gated=False,
    ):
        super(FeedForward, self).__init__()
        self.embedding_dim = embedding_dim
        self.forward_expansion = forward_expansion

        # Enhancement proposed in paper: GLU Variants Improve Transformer
        self.is_gated = is_gated
        if is_gated:
            self.linear_gated = nn.Linear(
                embedding_dim, forward_expansion * embedding_dim
            )

        self.linear1 = nn.Linear(embedding_dim, forward_expansion * embedding_dim)
        self.activation = Activation.from_str(activation).module()
        self.linear2 = nn.Linear(forward_expansion * embedding_dim, embedding_dim)
        self.dropout = nn.Dropout(dropout)

    def forward(self, x):
        g = self.activation(self.linear1(x))
        if self.is_gated:
            out = g * self.linear_gated(x)
        else:
            out = g
        return self.linear2(self.dropout(out))
