import abc


class INormalizer(metaclass=abc.ABCMeta):
    @classmethod
    def __subclasshook__(cls, subclass):
        return (
            hasattr(subclass, "normalize")
            and callable(subclass.normalize)
            and hasattr(subclass, "denormalize")
            and callable(subclass.denormalize)
            or NotImplemented
        )

    @abc.abstractmethod
    def normalize(self, values):
        raise NotImplementedError

    @abc.abstractmethod
    def denormalize(self, values):
        raise NotImplementedError
