import numpy as np
from matplotlib import pyplot as plt
from scipy import signal

from src.utils.seed import temp_seed


def sine_wave(length=1000, sampling_rate=20, frequency=0.005, amplitude=1):
    x = np.linspace(0, length, length * sampling_rate)
    y = np.sin(2 * np.pi * frequency * x) * amplitude
    return x, y


def triangle_wave(length=1000, sampling_rate=20, frequency=0.005, amplitude=1):
    x = np.linspace(0, length, length * sampling_rate)
    y = signal.sawtooth(2 * np.pi * frequency * x, 0.5) * amplitude
    return x, y


def sawtooth_wave(length=1000, sampling_rate=20, frequency=0.005, amplitude=1):
    x = np.linspace(0, length, length * sampling_rate)
    y = signal.sawtooth(2 * np.pi * frequency * x) * amplitude
    return x, y


def square_wave(length=1000, sampling_rate=20, frequency=0.005, amplitude=1):
    x = np.linspace(0, length, length * sampling_rate)
    y = signal.square(2 * np.pi * frequency * x) * amplitude
    return x, y


def noise(length=100, sampling_rate=200, mean=0, std=1, seed=1):
    with temp_seed(seed):
        x_noise = np.linspace(0, length, length)
        y_noise = np.random.normal(mean, std, size=length)
        x = np.linspace(0, length, length * sampling_rate)
        y = np.interp(x, x_noise, y_noise)
    return x, y


def linear(length=1000, sampling_rate=20, a=0.001):
    x = np.linspace(0, length, length * sampling_rate)
    y = a * x
    return x, y


def exponential(length=1000, sampling_rate=20):
    x = np.linspace(0, length, length * sampling_rate)
    y = x * x
    return x, y


if __name__ == "__main__":
    x1, y1 = sine_wave()
    x2, y2 = triangle_wave()
    x3, y3 = sawtooth_wave()
    x4, y4 = square_wave()
    x5, y5 = noise()
    x6, y6 = linear()
    x7, y7 = exponential()

    plt.figure()
    plt.plot(x1, y1)
    plt.plot(x2, y2)
    plt.plot(x3, y3)
    plt.plot(x4, y4)
    plt.plot(x5, y5)
    plt.plot(x6, y6)
    plt.plot(x7, y7)
    plt.show()
