import random
import torch

from src.dataset.LibraDataset import LibraDataset, LibraCategory
from src.dataset.SyntheticDataset import SyntheticDataset, SeriesType
from src.utils.seed import temp_seed


def boostrap_single_synthetic_dataset(
    input_dim: int,
    output_dim: int,
    series_type: SeriesType,
    dataset_length: int,
    train_test_ratio: float,
    seed: int = 1,
):
    with temp_seed(seed):
        dataset = SyntheticDataset(
            series_type,
            x_length=input_dim,
            y_length=output_dim,
            length=dataset_length,
        )
        dataset_len = len(dataset)
        train_set_size = int(dataset_len * train_test_ratio)
        test_set_size = dataset_len - train_set_size
        train_set, test_set = torch.utils.data.random_split(
            dataset, [train_set_size, test_set_size]
        )

        # Join shuffle dataset
        train_set = [train_set[i] for i in range(len(train_set))]
        random.shuffle(train_set)
        test_set = [test_set[i] for i in range(len(test_set))]
        random.shuffle(test_set)

    return train_set, test_set


def boostrap_synthetic_dataset(
    input_dim: int,
    output_dim: int,
    dataset_length: int,
    train_test_ratio: float,
    seed: int = 1,
):
    with temp_seed(seed):
        dataset1 = SyntheticDataset(
            SeriesType.SIMPLE,
            x_length=input_dim,
            y_length=output_dim,
            length=dataset_length,
        )
        dataset1_len = len(dataset1)
        train_set_size1 = int(dataset1_len * train_test_ratio)
        test_set_size1 = dataset1_len - train_set_size1
        train_set1, test_set1 = torch.utils.data.random_split(
            dataset1, [train_set_size1, test_set_size1]
        )

        dataset2 = SyntheticDataset(
            SeriesType.COMPOSED,
            x_length=input_dim,
            y_length=output_dim,
            length=dataset_length,
        )
        dataset2_len = len(dataset2)
        train_set_size2 = int(dataset2_len * train_test_ratio)
        test_set_size2 = dataset2_len - train_set_size2
        train_set2, test_set2 = torch.utils.data.random_split(
            dataset2, [train_set_size2, test_set_size2]
        )

        dataset3 = SyntheticDataset(
            SeriesType.COMPOSED_WITH_NOISE,
            x_length=input_dim,
            y_length=output_dim,
            length=dataset_length,
        )
        dataset3_len = len(dataset3)
        train_set_size3 = int(dataset3_len * train_test_ratio)
        test_set_size3 = dataset3_len - train_set_size3
        train_set3, test_set3 = torch.utils.data.random_split(
            dataset3, [train_set_size3, test_set_size3]
        )

        # Join and shuffle datasets
        train_set = train_set1 + train_set2 + train_set3
        test_set = test_set1 + test_set2 + test_set3
        train_set = [train_set[i] for i in range(len(train_set))]
        random.shuffle(train_set)
        test_set = [test_set[i] for i in range(len(test_set))]
        random.shuffle(test_set)

    return train_set, test_set


def boostrap_libra_dataset(
    input_dim: int,
    output_dim: int,
    categories: list[LibraCategory],
    dataset_filter: list,
    min_training_examples: int,
    max_values_per_dataset: int,
    train_test_ratio: float,
    seed: int = 1,
):
    with temp_seed(seed):
        dataset = LibraDataset(
            categories,
            x_length=input_dim,
            y_length=output_dim,
            dataset_filter=dataset_filter,
            max_values_per_dataset=max_values_per_dataset,
            min_training_examples_available=min_training_examples,
        )
        dataset_len = len(dataset)
        train_set_size = int(dataset_len * train_test_ratio)
        test_set_size = dataset_len - train_set_size
        train_set, test_set = torch.utils.data.random_split(
            dataset, [train_set_size, test_set_size]
        )

        # Shuffle datasets!
        train_set = [train_set[i] for i in range(len(train_set))]
        random.shuffle(train_set)
        test_set = [test_set[i] for i in range(len(test_set))]
        random.shuffle(test_set)

    return train_set, test_set
