import unittest
import torch

from src.components.activation.Activation import Activation
from src.evolutionary.core.conversion import create_phenotype_module
from src.evolutionary.genotype.GenotypeModule import GenotypeModule
from src.evolutionary.genotype.modules.ModuleCategory import ModuleCategory
from src.evolutionary.genotype.parameters.RangeParameter import RangeParameter
from src.evolutionary.genotype.parameters.SetParameter import SetParameter
from src.utils.helpers import save_json


def save_as_json(path):
    dicts = [m.to_dict() for m in modules]
    save_json(path, dicts)


modules = [
    GenotypeModule(
        "src.components.feed_forward.SingleFeedForward",
        {},
        ModuleCategory.FEED_FORWARD.value,
    ),
    GenotypeModule(
        "src.components.transformer.FeedForward",
        {
            "forward_expansion": RangeParameter(1, (1, 8)),
            "activation": SetParameter(
                Activation.ReLU.value, Activation.module_string_list()
            ),
            "dropout": RangeParameter(0.1, (0.0, 0.3), 7),
        },
        ModuleCategory.FEED_FORWARD.value,
    ),
]


class TestFeedForwardModules(unittest.TestCase):
    def test_instantiation_of_all_modules(self):
        batch_size = 16
        sequence_length = 200
        embedding_dim = 16
        dropout = 0.0

        shared_params = {
            "embedding_dim": embedding_dim,
            "input_dim": sequence_length,
            "dropout": dropout,
        }
        input_tensor = torch.rand(batch_size, sequence_length, embedding_dim)

        phenotype_modules = []
        output_tensors = []
        for module in modules:
            phenotype_module = create_phenotype_module(
                module, shared_params=shared_params
            )
            phenotype_modules.append(phenotype_module)
            output_tensor = phenotype_module(input_tensor)
            output_tensors.append(output_tensor)

        self.assertEqual(len(phenotype_modules), len(modules))
        for t in output_tensors:
            self.assertEqual(t.shape, (batch_size, sequence_length, embedding_dim))
