import unittest

from src.components.activation.Activation import Activation
from src.components.attention.preprocess.QKVPreprocess import QKVPreprocess
from src.components.attention.preprocess.ScoresPreprocess import ScoresPreprocess
from src.evolutionary.core.conversion import create_phenotype_module
from src.evolutionary.genotype.GenotypeModule import GenotypeModule
from src.evolutionary.genotype.modules.ModuleCategory import ModuleCategory
from src.evolutionary.genotype.parameters.RangeParameter import RangeParameter
from src.evolutionary.genotype.parameters.SetParameter import SetParameter


modules = [
    GenotypeModule(
        "src.components.attention.ScaledDotProductMultiHeadAttention",
        {
            "head_count": SetParameter(4, [1, 2, 4, 8]),
            "q_preprocess": SetParameter(
                QKVPreprocess.NoPreprocess.value, QKVPreprocess.module_string_list()
            ),
            "k_preprocess": SetParameter(
                QKVPreprocess.NoPreprocess.value, QKVPreprocess.module_string_list()
            ),
            "v_preprocess": SetParameter(
                QKVPreprocess.NoPreprocess.value, QKVPreprocess.module_string_list()
            ),
            "scores_preprocess": SetParameter(
                ScoresPreprocess.NoPreprocess.value,
                ScoresPreprocess.module_string_list(),
            ),
        },
        ModuleCategory.ATTENTION.value,
    ),
    GenotypeModule(
        "src.components.transformer.FeedForward",
        {
            "forward_expansion": RangeParameter(1, (1, 8)),
            "activation": SetParameter(
                Activation.ReLU.value, Activation.module_string_list()
            ),
            "dropout": RangeParameter(0.1, (0.0, 0.3), 7),
        },
        ModuleCategory.FEED_FORWARD.value,
    ),
    GenotypeModule(
        "src.components.positional_encoding.ape.SinusoidalAbsolutePositionalEncoding",
        {
            "dropout": RangeParameter(0.1, (0.0, 0.3), 7),
        },
        ModuleCategory.POSITIONAL_ENCODING.value,
    ),
    GenotypeModule(
        "src.components.normalization.LayerNormalization",
        {},
        ModuleCategory.NORMALIZATION.value,
    ),
]


class TestVanillaModules(unittest.TestCase):
    def test_instantiation_of_all_modules(self):
        shared_params = {"embedding_dim": 16, "dropout": 0.1}
        phenotype_modules = []
        for module in modules:
            phenotype_module = create_phenotype_module(
                module, shared_params=shared_params
            )
            phenotype_modules.append(phenotype_module)
        self.assertEqual(len(phenotype_modules), len(modules))
