import abc


class IParameter(metaclass=abc.ABCMeta):
    @classmethod
    def __subclasshook__(cls, subclass):
        return (
            hasattr(subclass, "get_value")
            and callable(subclass.get_value)
            and hasattr(subclass, "set_value")
            and callable(subclass.set_value)
            and hasattr(subclass, "get_allowed_values")
            and callable(subclass.get_allowed_values)
            and hasattr(subclass, "to_dict")
            and callable(subclass.to_dict)
            and hasattr(subclass, "get_type")
            and callable(subclass.get_type)
            or NotImplemented
        )

    @abc.abstractmethod
    def get_value(self):
        """Get parameter value"""
        raise NotImplementedError

    @abc.abstractmethod
    def set_value(self, value):
        """Set parameter value"""
        raise NotImplementedError

    @abc.abstractmethod
    def get_allowed_values(self) -> list:
        """Return a list of allowed values"""
        raise NotImplementedError

    @abc.abstractmethod
    def to_dict(self) -> dict:
        """Serialize to dictionary"""
        raise NotImplementedError

    def get_type(self):
        return self.__class__.__name__
