from enum import Enum

import torch
from torch import nn


class PhenotypeCellBranch(Enum):
    LEFT = 0
    RIGHT = 1


class PhenotypeCell(nn.Module):
    def __init__(self):
        super(PhenotypeCell, self).__init__()
        self._left_branch = nn.ModuleList()
        self._right_branch = nn.ModuleList()

    def add_module(self, branch: PhenotypeCellBranch, module):
        if branch is PhenotypeCellBranch.LEFT:
            self._left_branch.append(module)
        elif branch is PhenotypeCellBranch.RIGHT:
            self._right_branch.append(module)
        else:
            raise ValueError("Unknown branch")

    def forward(self, x_left: torch.Tensor, x_right: torch.Tensor):
        for module in self._left_branch:
            x_left = module(x_left)
        for module in self._right_branch:
            x_right = module(x_right)
        return x_left + x_right
