import copy

import torch.nn as nn
from src.components.transformer.ResidualConnection import ResidualConnection


class StandaloneDecoder(nn.Module):
    def __init__(self, decoder_layer, layer_count):
        super(StandaloneDecoder, self).__init__()
        self.layers = nn.ModuleList(
            [copy.deepcopy(decoder_layer) for _ in range(layer_count)]
        )
        self.norm = nn.LayerNorm(decoder_layer.embedding_dim)

    def forward(self, x, target_mask):
        for layer in self.layers:
            x = layer(x, target_mask)
        return self.norm(x)


class StandaloneDecoderLayer(nn.Module):
    def __init__(self, embedding_dim, self_attention, feed_forward, dropout):
        super(StandaloneDecoderLayer, self).__init__()
        self.embedding_dim = embedding_dim
        self.self_attention = self_attention
        self.feed_forward = feed_forward
        self.residual1 = ResidualConnection(embedding_dim, dropout)
        self.residual2 = ResidualConnection(embedding_dim, dropout)

    def forward(self, x, target_mask):
        x = self.residual1(x, lambda x: self.self_attention(x, x, x, target_mask))
        return self.residual2(x, self.feed_forward)
