import copy
import torch.nn as nn

from src.components.transformer.ResidualConnection import ResidualConnection


class StandaloneEncoder(nn.Module):
    def __init__(self, encoder_layer, layer_count):
        super(StandaloneEncoder, self).__init__()
        self.layers = nn.ModuleList(
            [copy.deepcopy(encoder_layer) for _ in range(layer_count)]
        )
        self.norm = nn.LayerNorm(encoder_layer.embedding_dim)

    def forward(self, x, mask):
        for layer in self.layers:
            x = layer(x, mask)
        return self.norm(x)

    def swap_attention(self, attention_module: nn.Module):
        for layer in self.layers:
            layer.swap_attention(copy.deepcopy(attention_module))


class StandaloneEncoderLayer(nn.Module):
    def __init__(self, embedding_dim, self_attention, feed_forward, dropout):
        super(StandaloneEncoderLayer, self).__init__()
        self.embedding_dim = embedding_dim
        self.self_attention = self_attention
        self.feed_forward = feed_forward
        self.residual1 = ResidualConnection(embedding_dim, dropout)
        self.residual2 = ResidualConnection(embedding_dim, dropout)

    def forward(self, x, mask):
        x = self.residual1(x, lambda x: self.self_attention(x, x, x, mask))
        return self.residual2(x, self.feed_forward)

    def swap_attention(self, attention_module: nn.Module):
        self.self_attention = attention_module
