import json

import numpy as np
import torch


def mean(lst: list):
    if len(lst) == 0:
        return float("nan")
    return sum(lst) / len(lst)


def is_nan(x):
    return x is np.nan or x != x


def sub_intervals_from_interval(a, b, n):
    return [a + x * (b - a) / n for x in range(n + 1)]


def subsequent_mask(size, device):
    attention_shape = (1, size, size)
    subseq_mask = np.triu(np.ones(attention_shape), k=1).astype("uint8")
    mask = torch.tensor(subseq_mask, dtype=torch.uint8) == 0
    return mask.to(device=device)


def namedtuple_fields_with_types(namedtuple: type) -> dict:
    fields = {}
    for k, v in list(namedtuple.__annotations__.items()):
        type_string = repr(v)
        type_string = type_string.replace("<class '", "").replace("'>", "")
        fields[k] = type_string
    return fields


def load_json(path):
    with open(path, "r") as f:
        data = json.load(f)
    return data


def save_json(path, data):
    with open(path, "w") as f:
        json.dump(data, f)
